@@ -79,6 +79,7 @@ authldap_free_config removed - no longer required.
 #include <errno.h>
 #include <pwd.h>
 #include <grp.h>
+#include <time.h>
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -169,7 +170,7 @@ static int ldapconncheck()
 static int read_env(const char *env, const char **copy,
 	const char *err, int needit, const char *value_default);
 static void copy_value(LDAP *ld, LDAPMessage *entry, const char *attribut,
-	char **copy);
+	char **copy, const char *username);
 
 /*
  * Function: read_env
@@ -372,7 +373,7 @@ static int authldap_read_config(struct ldap_info *ldap)
  *   void
  */
 static void copy_value(LDAP *ld, LDAPMessage *entry, const char *attribut,
-	char **copy)
+	char **copy, const char *username)
 {
   char ** values;
   values=ldap_get_values(ld,entry, (char *)attribut);
@@ -381,7 +382,7 @@ static void copy_value(LDAP *ld, LDAPMessage *entry, const char *attribut,
 	{
 #ifdef OPENLDAPV2
 	  int ld_errno = ldap_result2error(ld,entry,0);
-	  if (ld_errno != LDAP_DECODING_ERROR)
+	  if (ld_errno && ld_errno != LDAP_DECODING_ERROR)
 	    /* We didn't ask for this attribute */
 	    ldap_perror(ld,"ldap_get_values");
 #else
@@ -392,6 +393,16 @@ static void copy_value(LDAP *ld, LDAPMessage *entry, const char *attribut,
 		*copy=NULL;
 		return;
 	}
+  /* We accept only attribute with one value */
+	else if (ldap_count_values(values)>1)
+	 {
+		 *copy=strdup(values[0]);
+		 syslog(LOG_DAEMON,
+			"authldaplib: duplicate attribute %s for %s\n",
+			attribut,
+			username);
+		 *copy=NULL;
+	 }
   /* We accept only attribute with one value */
 	else if (ldap_count_values(values)!=1)
 	 {
@@ -631,14 +642,14 @@ char *quota=0;
   syslog(LOG_DAEMON|LOG_CRIT,"after ldap_first_entry\n");
 #endif
   /* Copy the directory and the password into struct */
-  copy_value(my_ldap_fp,entry,attributes[0],&homeDir);
+  copy_value(my_ldap_fp,entry,attributes[0],&homeDir, user);
   if (attributes[1])
-	  copy_value(my_ldap_fp,entry,attributes[1],&mailDir);
-  copy_value(my_ldap_fp,entry,attributes[2],&cn);
+	  copy_value(my_ldap_fp,entry,attributes[1],&mailDir, user);
+  copy_value(my_ldap_fp,entry,attributes[2],&cn, user);
   if (attributes[3])
-	  copy_value(my_ldap_fp,entry,attributes[3],&userPassword);
+	  copy_value(my_ldap_fp,entry,attributes[3],&userPassword, user);
   if (attributes[4])
-	  copy_value(my_ldap_fp,entry,attributes[4],&cryptPassword);
+	  copy_value(my_ldap_fp,entry,attributes[4],&cryptPassword, user);
 
 	au=my_ldap.uid;
 	ag=my_ldap.gid;
@@ -647,7 +658,7 @@ char *quota=0;
 	char *p=0;
 	unsigned long n;
 
-	copy_value(my_ldap_fp, entry, attributes[5], &p);
+	copy_value(my_ldap_fp, entry, attributes[5], &p, user);
 	if (!p)	return (0);
 
 
@@ -664,7 +675,7 @@ char *quota=0;
 	char *p=0;
 	unsigned long n;
 
-	copy_value(my_ldap_fp, entry, attributes[6], &p);
+	copy_value(my_ldap_fp, entry, attributes[6], &p, user);
 	if (!p)	return (0);
 
 	if (sscanf(p, "%lu", &n) > 0)
@@ -675,7 +686,7 @@ char *quota=0;
 #endif
   }
   if (attributes[8])
-	  copy_value(my_ldap_fp,entry,attributes[8],&quota);
+	  copy_value(my_ldap_fp,entry,attributes[8],&quota, user);
 
 	auth.sysusername=user;
 	auth.sysuserid= &au;
@@ -696,6 +707,13 @@ char *quota=0;
 
 	rc=0;
 
+	if (au == 0 || ag == 0)
+	{
+		syslog(LOG_DAEMON|LOG_CRIT, "authlib: refuse to authenticate %s: uid=%d, gid=%d\n", user, au, ag);
+		rc= 1;
+	}
+
+
 	if (pass)
 	{
 		if (my_ldap.authbind) 
