@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2000 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
@@ -14,11 +14,14 @@
 #if	HAVE_UNISTD_H
 #include	<unistd.h>
 #endif
-
+#if	HAVE_SHADOW_H
+#include	<shadow.h>
+#endif
 #include	"auth.h"
 #include	"authmod.h"
+#include	"authstaticlist.h"
 
-static const char rcsid[]="$Id: authshadow.c,v 1.7 2000/04/30 01:04:09 mrsam Exp $";
+static const char rcsid[]="$Id: authshadow.c,v 1.9 2000/10/14 17:34:44 mrsam Exp $";
 
 
 extern int auth_shadow_pre(const char *userid, const char *service,
@@ -99,3 +102,24 @@ int	rc;
 	}
 	return (ci.userret);
 }
+
+
+static void auth_shadow_cleanup()
+{
+#if	HAVE_ENDPWENT
+
+	endpwent();
+#endif
+
+#if	HAVE_ENDSPENT
+
+	endspent();
+#endif
+}
+
+struct authstaticinfo authshadow_info={
+	"authshadow",
+	auth_shadow,
+	auth_shadow_pre,
+	auth_shadow_cleanup,
+	0};
