@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 2000 Double Precision, Inc.  See COPYING for
+** Copyright 1998 - 2001 Double Precision, Inc.  See COPYING for
 ** distribution information.
 */
 
@@ -25,7 +25,7 @@ extern char *authvchkpw_isvirtual(char *);
 extern FILE *authvchkpw_file(const char *, const char *);
 extern struct passwd *authvchkpw_search(FILE *, const char *);
 
-static const char rcsid[]="$Id: preauthvchkpw.c,v 1.13 2001/02/09 13:23:35 mrsam Exp $";
+static const char rcsid[]="$Id: preauthvchkpw.c,v 1.15 2001/03/26 23:22:44 mrsam Exp $";
 
 int auth_vchkpw_pre(const char *userid, const char *service,
         int (*callback)(struct authinfo *, void *),
@@ -62,12 +62,20 @@ struct authinfo auth;
 		s = DEFAULT_DOMAIN;
 		notfound=EPERM;
 	}
-	
 
-	vget_assign(s,NULL,0,&uid, &gid);
-	pw=vauth_getpw(usercopy, s);
-	free(usercopy);
-	vclose();
+        vget_assign(s,NULL,0,&uid, &gid);
+        pw=vauth_getpw(usercopy, s);
+        if ( pw!=NULL ) {
+                if (pw->pw_dir == NULL || strlen(pw->pw_dir) == 0 ) {
+                        make_user_dir(usercopy, s, uid, gid);
+                        pw=vauth_getpw(usercopy, s);
+                }
+#if HAVE_VLOGAUTH
+                vlogauth(pw, s, service);
+#endif
+        }
+        free(usercopy);
+        vclose();
 
 	if (!pw)
 	{
