@@ -75,7 +75,7 @@
 
 #define	SHARED	"shared"
 
-static const char rcsid[]="$Id: imapd.c,v 1.56 2001/02/22 01:04:01 mrsam Exp $";
+static const char rcsid[]="$Id: imapd.c,v 1.63 2001/05/10 14:13:46 mrsam Exp $";
 
 extern time_t rfc822_parsedt(const char *);
 extern void fetchflags(unsigned long);
@@ -146,7 +146,7 @@ unsigned l;
 
 int is_draft(const char *m)
 {
-#if 0
+#if 1
 	/* Fix some PINE bugs first */
 
 	if (strcmp(m, "." DRAFTS))	return (0);
@@ -158,7 +158,7 @@ int is_draft(const char *m)
 
 int is_reserved(const char *m)
 {
-	if (is_trash(m) || is_draft(m))	return (1);
+	if (is_trash(m))	return (1);
 	return (0);
 }
 
@@ -338,7 +338,7 @@ int get_flagname(const char *p, struct imapflags *flags)
 		flags->seen=1;
 	else if (strcmp(p, "\\ANSWERED") == 0)
 		flags->answered=1;
-	else if (strcmp(p, "\\DRAFTS") == 0)
+	else if (strcmp(p, "\\DRAFT") == 0)
 		flags->drafts=1;
 	else if (strcmp(p, "\\DELETED") == 0)
 		flags->deleted=1;
@@ -373,8 +373,10 @@ const char *filename=mi->filename;
 		=flags->recent=flags->drafts=0;
 
 	if (is_draft(current_mailbox))
-		flags->drafts=1;
-
+	{
+		if (buf) strcat(buf, "\\Draft");
+		if (flags)  flags->drafts=1;
+	}
 	if ((filename=strrchr(filename, ':')) == 0
 		|| strncmp(filename, ":2,", 3))	return;
 
@@ -948,7 +950,7 @@ char	*p=malloc(strlen(mbox)+strlen(name)+sizeof("/cur/"));
 
 extern int doidle(time_t);
 
-static void idle(void)
+static void imapidle(void)
 {
 const char * envp = getenv("IMAP_IDLE_TIMEOUT");
 const int idleTimeout = envp ? atoi(envp) : 60;
@@ -1126,7 +1128,7 @@ FILE *oldfp;
 	free(newf);
 }
 
-// Hierarchical rename
+/* Hierarchical rename */
 
 static int chk_rename(const char *, const char *);
 static int do_rename(const char *, const char *);
@@ -1180,7 +1182,7 @@ static int scan_hier_rename(const char *oldname,
 		if (de->d_name[oldname_l] == 0)
 		{
 			if (ishier)
-				continue;	// Only the hierarchy
+				continue;	/* Only the hierarchy */
 
 			if ( (*func)(oldname, newname))
 			{
@@ -1218,7 +1220,7 @@ static int scan_hier_rename(const char *oldname,
 static int chk_rename(const char *on, const char *nn)
 {
 	if (access(nn, 0) == 0)
-		return (-1);	// Destination folder exists
+		return (-1);	/* Destination folder exists */
 	return (0);
 }
 
@@ -1262,7 +1264,7 @@ int	uid=0;
                if (current_mailbox)
                {
                        read_eol();
-                       idle();
+                       imapidle();
                        curtoken=nexttoken();
                        if (strcmp(curtoken->tokenbuf, "DONE") == 0)
                        {
@@ -1714,7 +1716,7 @@ int	uid=0;
 			return (0);
 		}
 
-		// Automatically UNSUBSCRIBE a DELETEd mailbox
+		/* Automatically UNSUBSCRIBE a DELETEd mailbox */
 
 		unsubscribe(mailbox_name);
 
