@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.
+** Copyright 1998 - 2001 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -28,7 +28,7 @@
 #include	"imaptoken.h"
 #include	"imapscanclient.h"
 
-static const char rcsid[]="$Id: search.c,v 1.16 2000/11/27 01:43:53 mrsam Exp $";
+static const char rcsid[]="$Id: search.c,v 1.17 2001/03/23 05:13:43 mrsam Exp $";
 
 extern time_t rfc822_parsedt(const char *);
 extern struct imapscaninfo current_mailbox_info;
@@ -189,10 +189,22 @@ unsigned long i, j;
 		{
 			j=0;
 			++msgset;
-			while (isdigit((int)(unsigned char)*msgset))
+			if (*msgset == '*')
 			{
-				j=j*10 + (*msgset++-'0');
+				++msgset;
+				/*
+				** Ok, we don't really need to know the upper
+				** end, just hack it.
+				*/
+				j=i;
+				if (j < n)
+					j=n;
 			}
+			else
+				while (isdigit((int)(unsigned char)*msgset))
+				{
+					j=j*10 + (*msgset++-'0');
+				}
 		}
 		if (n >= i && n <= j)	return (1);
 		if (*msgset == 0 || *msgset++ != ',')	break;
