@@ -10,7 +10,7 @@
 #include	"rfc1035mxlist.h"
 #include	"rfc1035_res.h"
 
-static const char rcsid[]="$Id: rfc1035mxlist.c,v 1.6 2000/05/28 16:58:28 mrsam Exp $";
+static const char rcsid[]="$Id: rfc1035mxlist.c,v 1.10 2000/09/08 00:00:32 mrsam Exp $";
 
 void rfc1035_mxlist_free(struct rfc1035_mxlist *p)
 {
@@ -29,10 +29,11 @@ static int addrecord(struct rfc1035_mxlist **list, const char *mxname,
 	int mxpreference,
 
 #if	RFC1035_IPV6
-	struct in6_addr *in
+	struct in6_addr *in,
 #else
-	struct in_addr *in
+	struct in_addr *in,
 #endif
+	int port
 	)
 {
 #if	RFC1035_IPV6
@@ -54,12 +55,12 @@ struct rfc1035_mxlist *p;
 #if	RFC1035_IPV6
 	sin.sin6_family=AF_INET6;
 	sin.sin6_addr= *in;
-	sin.sin6_port=htons(25);
+	sin.sin6_port=htons(port);
 	p->protocol=PF_INET6;
 #else
 	sin.sin_family=AF_INET;
 	sin.sin_addr= *in;
-	sin.sin_port=htons(25);
+	sin.sin_port=htons(port);
 	p->protocol=PF_INET;
 #endif
 
@@ -79,7 +80,7 @@ static int harvest_records(struct rfc1035_res *res,
 	struct rfc1035_reply *mxreply,
 	int mxpreference,
 	char *mxname,
-	int q_type, int *found, int autoquery);
+	int q_type, int *found, int autoquery, int port);
 
 #define	HARVEST_AUTOQUERY	1
 #define	HARVEST_NODUPE		2
@@ -87,7 +88,7 @@ static int harvest_records(struct rfc1035_res *res,
 static int add_arecords(struct rfc1035_res *res, struct rfc1035_mxlist **list,
 	struct rfc1035_reply *mxreply,
 	int mxpreference,
-	char *mxname)
+	char *mxname, int port)
 {
 #if	RFC1035_IPV6
 struct in6_addr	in;
@@ -103,7 +104,7 @@ int	rc;
 
 		rfc1035_ntoa(&in, buf);
 
-		if (addrecord(list, buf, mxpreference, &in))
+		if (addrecord(list, buf, mxpreference, &in, port))
 			return (RFC1035_MX_INTERNAL);
 
 		return (RFC1035_MX_OK);
@@ -126,29 +127,33 @@ records weren't found.
 	if (mxreply)
 	{
 		if ((rc=harvest_records(res, list, mxreply, mxpreference,
-			mxname, RFC1035_TYPE_AAAA, &found, 0)) != RFC1035_MX_OK)
+			mxname, RFC1035_TYPE_AAAA, &found, 0, port))
+				!= RFC1035_MX_OK)
 			return (rc);
 
 		if ((rc=harvest_records(res, list, mxreply, mxpreference,
-			mxname, RFC1035_TYPE_A, &found, HARVEST_NODUPE))
+			mxname, RFC1035_TYPE_A, &found, HARVEST_NODUPE,
+				port))
 						!= RFC1035_MX_OK)
 			return (rc);
 		if (found)	return (RFC1035_MX_OK);
 	}
 
 	if ((rc=harvest_records(res, list, mxreply, mxpreference, mxname,
-		RFC1035_TYPE_A, &found, HARVEST_AUTOQUERY|HARVEST_NODUPE))
+		RFC1035_TYPE_A, &found, HARVEST_AUTOQUERY|HARVEST_NODUPE, port))
 						!= RFC1035_MX_OK)
 		return (rc);
 	if (found)	return (RFC1035_MX_OK);
 
 	if ((rc=harvest_records(res, list, mxreply, mxpreference, mxname,
-		RFC1035_TYPE_AAAA, &found, HARVEST_AUTOQUERY)) != RFC1035_MX_OK)
+		RFC1035_TYPE_AAAA, &found, HARVEST_AUTOQUERY, port))
+			!= RFC1035_MX_OK)
 		return (rc);
 
 #else
 	if ((rc=harvest_records(res, list, mxreply, mxpreference, mxname,
-		RFC1035_TYPE_A, &found, HARVEST_AUTOQUERY)) != RFC1035_MX_OK)
+		RFC1035_TYPE_A, &found, HARVEST_AUTOQUERY, port))
+			!= RFC1035_MX_OK)
 		return (rc);
 #endif
 
@@ -162,7 +167,7 @@ static int harvest_records(struct rfc1035_res *res,
 	int mxpreference,
 	char *mxname,
 	int q_type, int *found,
-	int flags)
+	int flags, int port)
 {
 struct rfc1035_reply *areply=0;
 int index;
@@ -174,10 +179,13 @@ struct in_addr in;
 
 	index= -1;
 
-	if (!mxreply || ((index=rfc1035_replysearch_all( mxreply, mxname,
+	if (!mxreply || (
+		((index=rfc1035_replysearch_all( mxreply, mxname,
 					q_type,
 					RFC1035_CLASS_IN,
-					0)) < 0 && (flags & HARVEST_AUTOQUERY))
+					0)) < 0 ||
+			mxreply->allrrs[index]->rrtype != q_type)
+		&& (flags & HARVEST_AUTOQUERY))
 		)
 	{
 		index=rfc1035_resolve_cname(res,
@@ -235,7 +243,7 @@ struct in_addr in;
 				memcpy(&sin6, &q->address, sizeof(sin6));
 
 				if (memcmp(&sin6.sin6_addr, &in, sizeof(in))
-					== 0)
+					== 0 && q->priority == mxpreference)
 					break;
 			}
 			if ((flags & HARVEST_NODUPE) && q)	continue;
@@ -246,7 +254,7 @@ struct in_addr in;
 		in.s_addr=mxreply->allrrs[index]->rr.inaddr.s_addr;
 #endif
 		*found=1;
-		if (addrecord(list, mxname, mxpreference, &in))
+		if (addrecord(list, mxname, mxpreference, &in, port))
 		{
 			if (areply)
 				rfc1035_replyfree(areply);
@@ -260,7 +268,7 @@ struct in_addr in;
 }
 
 static int domxlistcreate(struct rfc1035_res *res,
-	const char *q_name, struct rfc1035_mxlist **list)
+	const char *q_name, struct rfc1035_mxlist **list, int port)
 {
 char	namebuf[RFC1035_MAXNAMESIZE+1];
 struct	rfc1035_reply *replyp;
@@ -282,7 +290,7 @@ RFC1035_ADDR	in;
 		if (rfc1035_aton(namebuf+1, &in))
 			return (RFC1035_MX_HARDERR);
 
-		if (addrecord(list, q_name, -1, &in))
+		if (addrecord(list, q_name, -1, &in, port))
 			return (RFC1035_MX_INTERNAL);
 		return (RFC1035_MX_OK);
 	}
@@ -306,7 +314,7 @@ RFC1035_ADDR	in;
 		{
 			rfc1035_replyfree(replyp);
 			strcpy(namebuf, q_name);
-			return (add_arecords(res, list, 0, -1, namebuf));
+			return (add_arecords(res, list, 0, -1, namebuf, port));
 		}
 
 		rfc1035_replyfree(replyp);
@@ -329,7 +337,8 @@ RFC1035_ADDR	in;
 			continue;
 
 		switch (add_arecords(res, list, replyp,
-			replyp->allrrs[index]->rr.mx.preference, mxname)) {
+			replyp->allrrs[index]->rr.mx.preference, mxname,
+			port)) {
 		case	RFC1035_MX_SOFTERR:
 			rfc1035_replyfree(replyp);
 			return (RFC1035_MX_SOFTERR);
@@ -348,14 +357,14 @@ RFC1035_ADDR	in;
 }
 
 static int domxlistcreate2(struct rfc1035_res *res,
-	const char *q_name, struct rfc1035_mxlist **list)
+	const char *q_name, struct rfc1035_mxlist **list, int port)
 {
 char	*buf;
 int	rc;
 
 	if (strchr(q_name, '.') || strchr(q_name, ':') ||
 		!res->rfc1035_defaultdomain)
-		return (domxlistcreate(res, q_name, list));
+		return (domxlistcreate(res, q_name, list, port));
 
 	if ((buf=malloc(strlen(q_name)+
 		strlen(res->rfc1035_defaultdomain)+2)) == 0)
@@ -363,7 +372,30 @@ int	rc;
 
 	strcat(strcat(strcpy(buf, q_name), "."), res->rfc1035_defaultdomain);
 
-	rc=domxlistcreate(res, buf, list);
+	rc=domxlistcreate(res, buf, list, port);
+
+	free(buf);
+	return (rc);
+}
+
+static int domxlistcreate3(struct rfc1035_res *res,
+	const char *q_name, struct rfc1035_mxlist **list)
+{
+char	*buf;
+int	rc;
+const char *p;
+
+	p=strchr(q_name, ',');
+
+	if (p == 0)	return (domxlistcreate2(res, q_name, list, 25));
+
+	if ((buf=malloc(p-q_name+1)) == 0)
+		return (-1);
+
+	memcpy(buf, q_name, p-q_name);
+	buf[p-q_name]=0;
+
+	rc=domxlistcreate2(res, buf, list, atoi(p+1));
 
 	free(buf);
 	return (rc);
@@ -372,7 +404,7 @@ int	rc;
 int rfc1035_mxlist_create(struct rfc1035_res *res,
 	const char *q_name, struct rfc1035_mxlist **list)
 {
-int	rc=domxlistcreate2(res, q_name, list);
+int	rc=domxlistcreate3(res, q_name, list);
 
 	if (rc != RFC1035_MX_OK)
 	{
