@@ -9,7 +9,24 @@
 #include	<stdio.h>
 #include	<string.h>
 #include	<stdlib.h>
+#include	<ctype.h>
 #include	<netdb.h>
+#if HAVE_DIRENT_H
+#include <dirent.h>
+#define NAMLEN(dirent) strlen((dirent)->d_name)
+#else
+#define dirent direct
+#define NAMLEN(dirent) (dirent)->d_namlen
+#if HAVE_SYS_NDIR_H
+#include <sys/ndir.h>
+#endif
+#if HAVE_SYS_DIR_H
+#include <sys/dir.h>
+#endif
+#if HAVE_NDIR_H
+#include <ndir.h>
+#endif
+#endif
 #if	HAVE_UNISTD_H
 #include	<unistd.h>
 #endif
@@ -20,13 +37,18 @@
 #if	HAVE_SYS_TYPES_H
 #include	<sys/types.h>
 #endif
+#if	HAVE_SYS_STAT_H
+#include	<sys/stat.h>
+#endif
 #include	<sys/socket.h>
 #include	<arpa/inet.h>
+#define	DEBUG_SAFESTACK	1	/* For openssl 0.9.6 */
+
 #include	<openssl/ssl.h>
 #include	<openssl/err.h>
 #include	<sys/time.h>
 
-static const char rcsid[]="$Id: starttls.c,v 1.9 2001/02/11 20:34:14 mrsam Exp $";
+static const char rcsid[]="$Id: starttls.c,v 1.20 2001/04/18 01:48:46 mrsam Exp $";
 
 #ifndef NO_RSA
 static RSA *rsa_callback(SSL *, int, int);
@@ -43,8 +65,7 @@ int peer_verify_level=SSL_VERIFY_PEER;
 		/* SSL_VERIFY_PEER */
 		/* SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT */
 const char *peer_cert_dir=0;
-const char *ourcacert=0;
-int okselfsignedcert=0;
+const char *peer_cert_file=0;
 
 /* Command-line options: */
 const char *clienthost=0;
@@ -53,13 +74,18 @@ const char *clientport=0;
 const char *server=0;
 const char *localfd=0;
 const char *remotefd=0;
+const char *statusfd=0;
 const char *tcpd=0;
 const char *peer_verify_domain=0;
+const char *verify_fail_msg=0;
+const char *fdprotocol=0;
 int peer_domain_verified;
 
 const char *printx509=0;
 FILE *printx509_fp;
 
+FILE *errfp;
+
 /* -------------------------------------------------------------------- */
 
 static int ssl_verify_callback(int, X509_STORE_CTX *);
@@ -69,7 +95,12 @@ static void sslerror(const char *pfix)
 char errmsg[256];
 
 	ERR_error_string(ERR_get_error(), errmsg);
-	fprintf(stderr, "starttls: %s: %s\n", pfix, errmsg);
+	fprintf(errfp, "starttls: %s: %s\n", pfix, errmsg);
+}
+
+static void nonsslerror(const char *pfix)
+{
+	fprintf(errfp, "%s: %s\n", pfix, strerror(errno));
 }
 
 SSL_CTX *create_tls(int isserver)
@@ -86,7 +117,7 @@ SSL_CTX *ctx;
 
 	if (!ctx)
 	{
-		perror("SSL_CTX_NEW");
+		nonsslerror("SSL_CTX_NEW");
 		return (0);
 	}
 	SSL_CTX_set_options(ctx, SSL_OP_ALL);
@@ -117,13 +148,15 @@ SSL_CTX *ctx;
 					cert_done=1;
 					DH_free(dh);
 				}
+				else
+					sslerror(dhcertfile);
 				BIO_free(bio);
 			}
 			else
 				sslerror(dhcertfile);
 			if (!cert_done)
 			{
-				fprintf(stderr, "starttls: DH init failed!\n");
+				fprintf(errfp, "starttls: DH init failed!\n");
 				SSL_CTX_free(ctx);
 				return (0);
 			}
@@ -154,18 +187,77 @@ SSL_CTX *ctx;
 		}
 	}
 
-	if (peer_cert_dir || ourcacert)
+	if (peer_cert_dir || peer_cert_file)
 	{
 		if ((!SSL_CTX_set_default_verify_paths(ctx))
-			|| (!SSL_CTX_load_verify_locations(ctx, ourcacert,
+			|| (!SSL_CTX_load_verify_locations(ctx, peer_cert_file,
 				peer_cert_dir)))
 		{
 			sslerror(peer_cert_dir);
 			SSL_CTX_free(ctx);
 			return (0);
 		}
-		SSL_CTX_set_client_CA_list(ctx,
-				SSL_load_client_CA_file(ourcacert));
+
+		if (isserver && peer_cert_file)
+		{
+			SSL_CTX_set_client_CA_list(ctx,
+						   SSL_load_client_CA_file
+						   (peer_cert_file));
+		}
+
+		if (isserver && peer_cert_dir)
+		{
+			DIR *dirp;
+			struct dirent *de;
+			X509 *x;
+
+			dirp=opendir(peer_cert_dir);
+			while (dirp && (de=readdir(dirp)) != NULL)
+			{
+				const char *p;
+				char *q;
+				FILE *fp;
+
+				p=strrchr(de->d_name, '.');
+				if (!p[0] || !p[1])
+					continue;
+				while (*++p)
+				{
+					if (strchr("0123456789", *p) == NULL)
+						break;
+				}
+				if (*p)
+					continue;
+
+				q=malloc(strlen(peer_cert_dir)
+					 +strlen(de->d_name) + 4);
+				if (!q)
+				{
+					nonsslerror("malloc");
+					exit(1);
+				}
+
+				strcat(strcat(strcpy(q, peer_cert_dir),
+					      "/"), de->d_name);
+
+				fp=fopen(q, "r");
+				if (!fp)
+				{
+					nonsslerror(q);
+					exit(1);
+				}
+				free(q);
+
+				while ((x=PEM_read_X509(fp, NULL, NULL, NULL)))
+				{
+					SSL_CTX_add_client_CA(ctx,x);
+					X509_free(x);
+				}
+				fclose(fp);
+			}
+			if (dirp)
+				closedir(dirp);
+                }
 	}
 	SSL_CTX_set_verify(ctx, peer_verify_level, ssl_verify_callback);
 	return (ctx);
@@ -173,59 +265,75 @@ SSL_CTX *ctx;
 
 static int ssl_verify_callback(int goodcert, X509_STORE_CTX *x509)
 {
-char txt[256];
-
-	X509_NAME_oneline(X509_get_subject_name(x509->current_cert),
-		txt, sizeof(txt));
-
-	if (printx509)
-	{
-		fprintf(printx509_fp, "Subject: %s\n", txt);
-		fflush(printx509_fp);
-	}
-
-	if (goodcert && peer_verify_domain)
-	{
-	char	*p;
-
-		for (p=txt; (p=strtok(p, "/")) != 0; p=0)
-		{
-		int	pl;
-
-			if (strncasecmp(p, "CN=", 3))
-				continue;
-			p += 3;
-			if (*p == '*')
-			{
-			int	l;
-
-				pl=strlen(++p);
-				l=strlen(peer_verify_domain);
-
-				if (pl <= l &&
-					strcasecmp(peer_verify_domain+l-pl,
-						p) == 0)
-					peer_domain_verified=1;
-			}
-			else if (strcasecmp(peer_verify_domain, p) == 0)
-				peer_domain_verified=1;
-		}
-	}
-
-	if (!okselfsignedcert && !goodcert)
-	{
-		fprintf(stderr, "starttls: Bad certificate from %s\n", txt);
-		return (0);
-	}
-	return 1;
+	if (peer_verify_domain && !goodcert)
+			return (0);
+	return (1);
 }
 
 static RSA *rsa_callback(SSL *s, int export, int keylength)
 {
-	fprintf(stderr, "Called rsa_callback\n");
 	return (RSA_generate_key(keylength,RSA_F4,NULL,NULL));
 }
 
+static char domain[256];
+
+static void dump_x509(X509 *x509, FILE *printx509_fp)
+{
+	X509_NAME *subj=X509_get_subject_name(x509);
+	int nentries, j;
+
+	if (!subj)
+		return;
+
+	if (printx509_fp)
+		fprintf(printx509_fp, "Subject:\n");
+
+	nentries=X509_NAME_entry_count(subj);
+	for (j=0; j<nentries; j++)
+	{
+		const char *obj_name;
+		X509_NAME_ENTRY *e;
+		ASN1_OBJECT *o;
+		ASN1_STRING *d;
+
+		int dlen;
+		unsigned char *ddata;
+
+		e=X509_NAME_get_entry(subj, j);
+		if (!e)
+			continue;
+
+		o=X509_NAME_ENTRY_get_object(e);
+		d=X509_NAME_ENTRY_get_data(e);
+
+		if (!o || !d)
+			continue;
+
+		obj_name=OBJ_nid2sn(OBJ_obj2nid(o));
+
+		dlen=ASN1_STRING_length(d);
+		ddata=ASN1_STRING_data(d);
+
+		if (strcasecmp(obj_name, "CN") == 0)
+		{
+			if (dlen >= sizeof(domain)-1)
+				dlen=sizeof(domain)-1;
+
+			memcpy(domain, ddata, dlen);
+			domain[dlen]=0;
+		}
+
+		if (printx509_fp)
+		{
+			fprintf(printx509_fp, "   %s=", obj_name);
+			fwrite(ddata, dlen, 1, printx509_fp);
+			fprintf(printx509_fp, "\n");
+		}
+	}
+	if (printx509_fp)
+		fprintf(printx509_fp, "\n");
+}
+
 SSL *connect_tls(SSL_CTX *ctx, int isserver, int fd)
 {
 SSL *ssl;
@@ -247,7 +355,7 @@ SSL *ssl;
 		printx509_fp=fdopen(atoi(printx509), "w");
 		if (!printx509_fp)
 		{
-			perror("fdopen");
+			nonsslerror("fdopen");
 			printx509=0;
 		}
 	}
@@ -280,22 +388,74 @@ SSL *ssl;
 			ERR_remove_state(0);
 			if (printx509)
 				fclose(printx509_fp);
+
 			printx509=0;
 			return (0);
 		}
 	}
-	if (printx509)
-		fclose(printx509_fp);
-	printx509=0;
 
-	if (!peer_domain_verified)
 	{
-		SSL_set_shutdown(ssl, SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
-		SSL_free(ssl);
-		ERR_remove_state(0);
-		fprintf(stderr, "starttls: peer domain not verified.\n");
-		return (0);
+		STACK_OF(X509) *peer_cert_chain=SSL_get_peer_cert_chain(ssl);
+		int i;
+
+		domain[0]=0;
+		for (i=0; peer_cert_chain && i<peer_cert_chain->stack.num; i++)
+			dump_x509((X509 *)peer_cert_chain->stack.data[i],
+				  printx509 ? printx509_fp:NULL);
+		if (isserver)
+		{
+			X509 *x=SSL_get_peer_certificate(ssl);
+
+			if (x)
+				dump_x509(x, printx509 ? printx509_fp:NULL);
+		}
 	}
+
+	if (peer_verify_domain)
+	{
+		char	*p=domain;
+
+		if (*p == '*')
+		{
+			int	pl, l;
+
+			pl=strlen(++p);
+			l=strlen(peer_verify_domain);
+
+			if (*p == '.' && pl <= l &&
+			    strcasecmp(peer_verify_domain+l-pl, p) == 0)
+				peer_domain_verified=1;
+		}
+		else if (strcasecmp(peer_verify_domain, p) == 0)
+			peer_domain_verified=1;
+	}
+
+	if (printx509)
+	{
+		SSL_CIPHER *cipher;
+
+		cipher=SSL_get_current_cipher(ssl);
+
+		if (cipher)
+		{
+			const char *c;
+
+			c=SSL_CIPHER_get_name(cipher);
+
+			if (c)
+				fprintf(printx509_fp, "Cipher: %s\n", c);
+
+			c=SSL_CIPHER_get_version(cipher);
+			if (c)
+				fprintf(printx509_fp, "Version: %s\n", c);
+
+
+			fprintf(printx509_fp, "Bits: %d\n",
+				SSL_CIPHER_get_bits(cipher, NULL));
+		}
+		fclose(printx509_fp);
+	}
+	printx509=0;
 	return (ssl);
 }
 
@@ -311,35 +471,6 @@ void disconnect_tls(SSL_CTX *ctx, SSL *ssl)
 	SSL_CTX_free(ctx);
 }
 
-#if 0 
-static void mylog(const char *what, const char *buf, int cnt)
-{
-fprintf(stderr, "couriertls: %s: ", what);
-
-	for (; cnt; ++buf, --cnt)
-	{
-		if (*buf == '\n')
-		{
-			fprintf(stderr, "\\n");
-			continue;
-		}
-		if (*buf == '\r')
-		{
-			fprintf(stderr, "\\r");
-			continue;
-		}
-		if ((int)(unsigned char)*buf < ' ')
-		{
-			fprintf(stderr, "\\x%02X", (int)(unsigned char)*buf);
-			continue;
-		}
-		fprintf(stderr, "%c", *buf);
-	}
-	fprintf(stderr, "\n");
-	fflush(stderr);
-}
-#endif
-
 void transfer_tls(SSL *ssl, int sslfd, int stdinfd, int stdoutfd)
 {
 char	from_ssl_buf[BUFSIZ], to_ssl_buf[BUFSIZ];
@@ -352,7 +483,7 @@ int	suppress_write;
 
 	if (fcntl(sslfd, F_SETFL, O_NONBLOCK))
 	{
-		perror("fcntl");
+		nonsslerror("fcntl");
 		return;
 	}
 
@@ -418,7 +549,7 @@ int	suppress_write;
 
 		if (select(maxfd+1, &fdr, &fdw, 0, 0) <= 0)
 		{
-			perror("select");
+			nonsslerror("select");
 			return;
 		}
 
@@ -498,7 +629,7 @@ int	suppress_write;
 
 /* ----------------------------------------------------------------------- */
 
-static void prepsocket(int sockfd)
+static int prepsocket(int sockfd)
 {
 #ifdef  SO_KEEPALIVE
 	{
@@ -509,7 +640,8 @@ static void prepsocket(int sockfd)
 		if (setsockopt(sockfd, SOL_SOCKET, SO_KEEPALIVE,
 			(const char *)&dummy, sizeof(dummy)) < 0)
                 {
-                        perror("setsockopt");
+                        nonsslerror("setsockopt");
+			return (1);
                 }
 	}
 #endif
@@ -524,10 +656,12 @@ static void prepsocket(int sockfd)
 		if (setsockopt(sockfd, SOL_SOCKET, SO_LINGER,
 			(const char *)&l, sizeof(l)) < 0)
 		{
-			perror("setsockopt");
+			nonsslerror("setsockopt");
+			return (1);
 		}
 	}
 #endif
+	return (0);
 }
 
 static void startclient(int argn, int argc, char **argv, int fd,
@@ -546,12 +680,12 @@ int	streampipe[2];
 
 	if (s_pipe(streampipe))
 	{
-		perror("s_pipe");
+		nonsslerror("s_pipe");
 		exit(1);
 	}
 	if ((p=fork()) == -1)
 	{
-		perror("fork");
+		nonsslerror("fork");
 		close(streampipe[0]);
 		close(streampipe[1]);
 		exit(1);
@@ -572,14 +706,14 @@ int	streampipe[2];
 		argvec=malloc(sizeof(char *)*(argc-argn+1));
 		if (!argvec)
 		{
-			perror("malloc");
+			nonsslerror("malloc");
 			exit(1);
 		}
 		for (n=0; n<argc-argn; n++)
 			argvec[n]=argv[argn+n];
 		argvec[n]=0;
 		execvp(argvec[0], argvec);
-		perror(argvec[0]);
+		nonsslerror(argvec[0]);
 		exit(1);
 	}
 	close(streampipe[1]);
@@ -610,7 +744,7 @@ int	port_num;
 
 		if (!servent)
 		{
-			fprintf(stderr, "%s: invalid port.\n", port);
+			fprintf(errfp, "%s: invalid port.\n", port);
 			return (-1);
 		}
 		port_num=servent->s_port;
@@ -622,7 +756,7 @@ int	port_num;
 	{
 		if ((addrs=malloc(sizeof(addr))) == 0)
 		{
-			perror("malloc");
+			nonsslerror("malloc");
 			return (-1);
 		}
 		memcpy(addrs, &addr, sizeof(addr));
@@ -632,14 +766,14 @@ int	port_num;
 	{
 		if (rfc1035_a(&rfc1035_default_resolver, host, &addrs, &naddrs))
 		{
-			fprintf(stderr, "%s: not found.\n", host);
+			fprintf(errfp, "%s: not found.\n", host);
 			return (-1);
 		}
 	}
 
         if ((fd=rfc1035_mksocket(SOCK_STREAM, 0, &af)) < 0)
         {
-                perror("socket");
+                nonsslerror("socket");
                 return (-1);
         }
 
@@ -656,7 +790,7 @@ int	port_num;
 	if (n >= naddrs)
 	{
 		close(fd);
-		perror("connect");
+		nonsslerror("connect");
 		return (-1);
 	}
 
@@ -665,14 +799,51 @@ int	port_num;
 
 /* Connect to a remote server */
 
-static int dossl(int fd, int argn, int argc, char **argv)
+char *get_ip_certfile(const char *file, const char *ip) {
+	char *test_file = malloc(strlen(file)+strlen(ip)+2);
+
+	strcpy(test_file, file);
+	strcat(test_file, ".");
+	strcat(test_file, ip);
+
+	if (access(test_file, R_OK) == 0) {
+		return test_file;
+	} else {
+		free(test_file);
+		/* Check for ipv4 version if ip is ipv6 */
+		if (strncmp(ip, "::ffff:", 7) == 0 && strchr(ip, '.')) {
+			return get_ip_certfile(file, ip+7);
+		}
+	}
+
+	return NULL;
+}
+
+static int dossl(int fd, int argn, int argc, char **argv, FILE *statusfp)
 {
 SSL_CTX *ctx;
 SSL	*ssl;
 
 int	stdin_fd, stdout_fd;
+const char *ip = NULL;
 
-	prepsocket(fd);
+	if ((ip = getenv("TCPLOCALIP")) != NULL) {
+		if (certfile) {
+			const char *newcertfile =
+				get_ip_certfile(certfile, ip);
+			if (newcertfile)
+				certfile = newcertfile;
+		}
+		if (dhcertfile) {
+			const char *newdhcertfile =
+				get_ip_certfile(dhcertfile, ip);
+			if (newdhcertfile)
+				dhcertfile = newdhcertfile;
+		}
+	}
+	
+	if (prepsocket(fd))
+		return (1);
 
 	ctx=create_tls(server ? 1:0);
 	if (ctx == 0)	return (1);
@@ -689,6 +860,25 @@ int	stdin_fd, stdout_fd;
 
 	startclient(argn, argc, argv, fd, &stdin_fd, &stdout_fd);
 
+	if (!peer_domain_verified)
+	{
+		SSL_set_shutdown(ssl, SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
+		SSL_free(ssl);
+		ERR_remove_state(0);
+		if (verify_fail_msg)
+			fprintf(statusfp, "%s", verify_fail_msg);
+		else
+			fprintf(errfp,
+				"starttls: unable to verify peer domain.\n");
+		return (0);
+	}
+
+	if (statusfp)
+	{
+		fclose(statusfp);
+		errfp=stderr;
+	}
+
 	transfer_tls(ssl, fd, stdin_fd, stdout_fd);
 	disconnect_tls(ctx, ssl);
 	return (0);
@@ -702,12 +892,171 @@ const char *v=getenv(n);
 	return (v);
 }
 
+struct protoreadbuf {
+	char buffer[512];
+	char *bufptr;
+	int bufleft;
+
+	char line[256];
+} ;
+
+#define PRB_INIT(p) ( (p)->bufptr=0, (p)->bufleft=0)
+
+static char protoread(int fd, struct protoreadbuf *prb)
+{
+	fd_set fds;
+	struct timeval tv;
+
+	FD_ZERO(&fds);
+	FD_SET(fd, &fds);
+
+	tv.tv_sec=60;
+	tv.tv_usec=0;
+
+	if (select(fd+1, &fds, NULL, NULL, &tv) <= 0)
+	{
+		nonsslerror("select");
+		exit(1);
+	}
+
+	if ( (prb->bufleft=read(fd, prb->buffer, sizeof(prb->buffer))) <= 0)
+	{
+		errno=ECONNRESET;
+		nonsslerror("read");
+		exit(1);
+	}
+
+	prb->bufptr= prb->buffer;
+
+	--prb->bufleft;
+	return (*prb->bufptr++);
+}
+
+#define PRB_GETCH(fd,prb) ( (prb)->bufleft-- > 0 ? *(prb)->bufptr++:\
+				protoread( (fd), (prb)))
+
+static const char *prb_getline(int fd, struct protoreadbuf *prb)
+{
+	int i=0;
+	char c;
+
+	while ((c=PRB_GETCH(fd, prb)) != '\n')
+	{
+		if ( i < sizeof (prb->line)-1)
+			prb->line[i++]=c;
+	}
+	prb->line[i]=0;
+	return (prb->line);
+}
+
+static void prb_write(int fd, struct protoreadbuf *prb, const char *p)
+{
+	printf("%s", p);
+	while (*p)
+	{
+		int l=write(fd, p, strlen(p));
+
+		if (l <= 0)
+		{
+			nonsslerror("write");
+			exit(1);
+		}
+		p += l;
+	}
+}
+
+static int goodimap(const char *p)
+{
+	if (*p == 'x' && p[1] && isspace((int)(unsigned char)p[1]))
+		++p;
+	else
+	{
+		if (*p != '*')
+			return (0);
+		++p;
+	}
+	while (*p && isspace((int)(unsigned char)*p))
+		++p;
+	if (strncasecmp(p, "BAD", 3) == 0)
+	{
+		exit(1);
+	}
+
+	if (strncasecmp(p, "BYE", 3) == 0)
+	{
+		exit(1);
+	}
+
+	if (strncasecmp(p, "NO", 2) == 0)
+	{
+		exit(1);
+	}
+
+	return (strncasecmp(p, "OK", 2) == 0);
+}
+
+static void imap_proto(int fd)
+{
+	struct protoreadbuf prb;
+	const char *p;
+
+	PRB_INIT(&prb);
+
+	do
+	{
+		p=prb_getline(fd, &prb);
+		printf("%s\n", p);
+
+	} while (!goodimap(p));
+
+	prb_write(fd, &prb, "x STARTTLS\r\n");
+
+	do
+	{
+		p=prb_getline(fd, &prb);
+		printf("%s\n", p);
+	} while (!goodimap(p));
+}
+
+static void smtp_proto(int fd)
+{
+	struct protoreadbuf prb;
+	const char *p;
+
+	PRB_INIT(&prb);
+
+	do
+	{
+		p=prb_getline(fd, &prb);
+		printf("%s\n", p);
+	} while ( ! ( isdigit((int)(unsigned char)p[0]) && 
+		      isdigit((int)(unsigned char)p[1]) &&
+		      isdigit((int)(unsigned char)p[2]) &&
+		      (p[3] == 0 || isspace((int)(unsigned char)p[3]))));
+	if (strchr("123", *p) == 0)
+		exit(1);
+
+	prb_write(fd, &prb, "STARTTLS\r\n");
+
+	do
+	{
+		p=prb_getline(fd, &prb);
+		printf("%s\n", p);
+	} while ( ! ( isdigit((int)(unsigned char)p[0]) && 
+		      isdigit((int)(unsigned char)p[1]) &&
+		      isdigit((int)(unsigned char)p[2]) &&
+		      (p[3] == 0 || isspace((int)(unsigned char)p[3]))));
+	if (strchr("123", *p) == 0)
+		exit(1);
+
+}
+
 int main(int argc, char **argv)
 {
 const char *s;
 int	argn;
 int	fd;
-
+struct stat stat_buf;
 static struct args arginfo[] = {
 	{ "host", &clienthost },
 	{ "localfd", &localfd},
@@ -717,8 +1066,15 @@ static struct args arginfo[] = {
 	{ "server", &server},
 	{ "tcpd", &tcpd},
 	{ "verify", &peer_verify_domain},
+	{ "verifyfailmsg", &verify_fail_msg},
+	{ "statusfd", &statusfd},
+	{ "protocol", &fdprotocol},
 	{0}};
+FILE	*statusfp=0;
+ void (*protocol_func)(int)=0;
 
+	errfp=stderr;
+ 
 	s=safe_getenv("TLS_PROTOCOL");
 	if (*s) protocol=s;
 
@@ -751,18 +1107,37 @@ static struct args arginfo[] = {
 		break;
 	}
 
-	s=safe_getenv("TLS_PEERCERTDIR");
-	if (*s)	peer_cert_dir=s;
-
-	s=safe_getenv("TLS_OURCACERT");
-
-	if (*s)	ourcacert=s;
-
-	s=safe_getenv("TLS_ALLOWSELFSIGNEDCERT");
-	okselfsignedcert=atoi(s);
+	s=safe_getenv("TLS_TRUSTCERTS");
+	if (s && stat(s, &stat_buf) == 0)
+	{
+		if (S_ISDIR(stat_buf.st_mode))
+			peer_cert_dir=s;
+		else
+			peer_cert_file=s;
+	}
 
 	argn=argparse(argc, argv, arginfo);
 
+	if (statusfd)
+		statusfp=fdopen(atoi(statusfd), "w");
+
+	if (statusfp)
+		errfp=statusfp;
+
+	if (fdprotocol)
+	{
+		if (strcmp(fdprotocol, "smtp") == 0)
+			protocol_func= &smtp_proto;
+		else if (strcmp(fdprotocol, "imap") == 0)
+			protocol_func= &imap_proto;
+		else
+		{
+			fprintf(stderr, "--protocol=%s - unknown protocol.\n",
+				fdprotocol);
+			exit(1);
+		}
+	}
+
 	if (tcpd)
 	{
 		close(1);
@@ -775,12 +1150,12 @@ static struct args arginfo[] = {
 		fd=connectremote(clienthost, clientport);
 	else
 	{
-		fprintf(stderr, "%s: specify remote location.\n", argv[0]);
+		fprintf(errfp, "%s: specify remote location.\n", argv[0]);
 		return (1);
 	}
 
 	if (fd < 0)	return (1);
-
-	return (dossl(fd, argn, argc, argv));
+	if (protocol_func)
+		(*protocol_func)(fd);
+	return (dossl(fd, argn, argc, argv, statusfp));
 }
-
